package com.sun.jndi.ldap;

import javax.naming.NamingException;
import java.util.Iterator;
import java.util.ArrayList;
import com.sun.jndi.ldap.spi.LdapDnsProviderResult;
import java.util.Hashtable;
import java.security.AccessController;
import sun.security.util.SecurityConstants;
import java.security.Permission;
import com.sun.jndi.ldap.spi.LdapDnsProvider;
import java.util.ServiceLoader;

final class LdapDnsProviderService
{
    private static volatile LdapDnsProviderService service;
    private static final Object LOCK;
    private final ServiceLoader<LdapDnsProvider> providers;
    
    private LdapDnsProviderService() {
        if (System.getSecurityManager() == null) {
            this.providers = ServiceLoader.load(LdapDnsProvider.class, ClassLoader.getSystemClassLoader());
        }
        else {
            this.providers = (ServiceLoader)AccessController.doPrivileged(()/*  */ -> ServiceLoader.load(LdapDnsProvider.class, ClassLoader.getSystemClassLoader()), null, new Permission[] { new RuntimePermission("ldapDnsProvider"), SecurityConstants.GET_CLASSLOADER_PERMISSION });
        }
    }
    
    static LdapDnsProviderService getInstance() {
        if (LdapDnsProviderService.service != null) {
            return LdapDnsProviderService.service;
        }
        synchronized (LdapDnsProviderService.LOCK) {
            if (LdapDnsProviderService.service != null) {
                return LdapDnsProviderService.service;
            }
            LdapDnsProviderService.service = new LdapDnsProviderService();
        }
        return LdapDnsProviderService.service;
    }
    
    LdapDnsProviderResult lookupEndpoints(String s, Hashtable<?, ?> t) throws NamingException {
        Iterator<LdapDnsProvider> iterator;
        Hashtable hashtable;
        LdapDnsProviderResult ldapDnsProviderResult;
        for (iterator = this.providers.iterator(), hashtable = new Hashtable(t), ldapDnsProviderResult = null; ldapDnsProviderResult == null && iterator.hasNext(); ldapDnsProviderResult = (LdapDnsProviderResult)((LdapDnsProvider)iterator.next()).lookupEndpoints(s, hashtable).filter(ldapDnsProviderResult2/* com.sun.jndi.ldap.spi.LdapDnsProviderResult, */ -> ldapDnsProviderResult2.getEndpoints().size() > 0).orElse(null)) {}
        if (ldapDnsProviderResult == null) {
            return (LdapDnsProviderResult)new DefaultLdapDnsProvider().lookupEndpoints(s, t).orElse(new LdapDnsProviderResult("", new ArrayList()));
        }
        return ldapDnsProviderResult;
    }
    
    static {
        LOCK = new int[0];
    }
}
